@@ -1,8 +1,8 @@
-use binread::NullString;
+use binread::BinRead;
 use std::io::{Cursor, Seek, SeekFrom, Write};
 
 use crate::{
-    anim::*, formats::mesh::*, matl::*, shdr::*, skel::*, CString, Ptr64, RelPtr64, SsbhArray,
+    anim::*, formats::mesh::*, matl::*, shdr::*, skel::*, Offset, Ptr, RelPtr64, SsbhArray,
     SsbhByteBuffer, SsbhFile, SsbhWrite,
 };
 
@@ -42,6 +42,10 @@ macro_rules! ssbh_write_c_enum_impl {
             fn size_in_bytes(&self) -> u64 {
                 std::mem::size_of::<$underlying_type>() as u64
             }
+
+            fn alignment_in_bytes() -> u64 {
+                std::mem::align_of::<$underlying_type>() as u64
+            }
         }
     };
 }
@@ -72,60 +76,6 @@ ssbh_write_c_enum_impl!(ParamId, u64);
 
 ssbh_write_c_enum_impl!(BillboardType, u8);
 
-macro_rules! ssbh_write_impl {
-    ($($id:ident),*) => {
-        $(
-            impl SsbhWrite for $id {
-                fn ssbh_write<W: std::io::Write + std::io::Seek>(
-                    &self,
-                    writer: &mut W,
-                    _data_ptr: &mut u64,
-                ) -> std::io::Result<()> {
-                    writer.write_all(&self.to_le_bytes())?;
-                    Ok(())
-                }
-
-                fn size_in_bytes(&self) -> u64 {
-                    std::mem::size_of::<Self>() as u64
-                }
-            }
-        )*
-    }
-}
-
-ssbh_write_impl!(u8, u16, u32, u64, u128, i8, i16, i32, i64, i128, f32, f64);
-
-impl<T: binread::BinRead + SsbhWrite> SsbhWrite for Option<T> {
-    fn ssbh_write<W: Write + Seek>(
-        &self,
-        writer: &mut W,
-        data_ptr: &mut u64,
-    ) -> std::io::Result<()> {
-        match self {
-            Some(value) => value.ssbh_write(writer, data_ptr),
-            None => Ok(()),
-        }
-    }
-
-    fn size_in_bytes(&self) -> u64 {
-        // None values are skipped entirely.
-        // TODO: Is this a reasonable implementation?
-        match self {
-            Some(value) => value.size_in_bytes(),
-            None => 0u64,
-        }
-    }
-
-    fn alignment_in_bytes(&self) -> u64 {
-        // Use the underlying type's alignment.
-        // This is a bit of a hack since None values won't be written anyway.
-        match self {
-            Some(value) => value.alignment_in_bytes(),
-            None => 8,
-        }
-    }
-}
-
 fn write_array_header<W: Write + Seek>(
     writer: &mut W,
     data_ptr: &mut u64,
@@ -152,8 +102,8 @@ impl SsbhWrite for SsbhByteBuffer {
         data_ptr: &mut u64,
     ) -> std::io::Result<()> {
         let current_pos = writer.stream_position()?;
-        if *data_ptr <= current_pos {
-            *data_ptr += self.size_in_bytes();
+        if *data_ptr < current_pos + self.size_in_bytes() {
+            *data_ptr = current_pos + self.size_in_bytes();
         }
 
         write_array_header(writer, data_ptr, self.elements.len())?;
@@ -174,43 +124,16 @@ impl SsbhWrite for SsbhByteBuffer {
     }
 }
 
-impl<T: SsbhWrite + binread::BinRead> SsbhWrite for &[T] {
-    fn ssbh_write<W: Write + Seek>(
-        &self,
-        writer: &mut W,
-        data_ptr: &mut u64,
-    ) -> std::io::Result<()> {
-        // The data pointer must point past the containing struct.
-        let current_pos = writer.stream_position()?;
-        if *data_ptr <= current_pos {
-            *data_ptr = current_pos + self.size_in_bytes();
-        }
-
-        for element in self.iter() {
-            element.ssbh_write(writer, data_ptr)?;
-        }
-
-        Ok(())
-    }
-
-    fn size_in_bytes(&self) -> u64 {
-        // TODO: This won't work for Vec<Option<T>> since only the first element is checked.
-        match self.first() {
-            Some(element) => self.len() as u64 * element.size_in_bytes(),
-            None => 0,
-        }
-    }
-}
-
 impl<T: binread::BinRead + SsbhWrite + Sized> SsbhWrite for SsbhArray<T> {
     fn ssbh_write<W: Write + Seek>(
         &self,
         writer: &mut W,
         data_ptr: &mut u64,
     ) -> std::io::Result<()> {
+        // TODO: Create a macro or function for this?
         let current_pos = writer.stream_position()?;
-        if *data_ptr <= current_pos {
-            *data_ptr += self.size_in_bytes();
+        if *data_ptr < current_pos + self.size_in_bytes() {
+            *data_ptr = current_pos + self.size_in_bytes();
         }
 
         write_array_header(writer, data_ptr, self.elements.len())?;
@@ -230,39 +153,12 @@ impl<T: binread::BinRead + SsbhWrite + Sized> SsbhWrite for SsbhArray<T> {
         16
     }
 
-    fn alignment_in_bytes(&self) -> u64 {
+    fn alignment_in_bytes() -> u64 {
         // Arrays are always 8 byte aligned.
         8
     }
 }
 
-impl SsbhWrite for NullString {
-    fn ssbh_write<W: Write + Seek>(
-        &self,
-        writer: &mut W,
-        _data_ptr: &mut u64,
-    ) -> std::io::Result<()> {
-        if self.len() == 0 {
-            // Handle empty strings.
-            writer.write_all(&[0u8; 4])?;
-        } else {
-            // Write the data and null terminator.
-            writer.write_all(&self)?;
-            writer.write_all(&[0u8])?;
-        }
-        Ok(())
-    }
-
-    fn size_in_bytes(&self) -> u64 {
-        // Include the null byte in the length.
-        self.len() as u64 + 1
-    }
-
-    fn alignment_in_bytes(&self) -> u64 {
-        4
-    }
-}
-
 fn write_rel_ptr_aligned_specialized<
     W: Write + Seek,
     T,
@@ -324,70 +220,62 @@ fn write_ssbh_header<W: Write + Seek>(writer: &mut W, magic: &[u8; 4]) -> std::i
     Ok(())
 }
 
-impl<const N: usize> SsbhWrite for CString<N> {
-    fn ssbh_write<W: std::io::Write + std::io::Seek>(
-        &self,
-        writer: &mut W,
-        _data_ptr: &mut u64,
-    ) -> std::io::Result<()> {
-        if self.0 .0.len() == 0 {
-            // Handle empty strings.
-            writer.write_all(&[0u8; N])?;
-        } else {
-            // Write the data and null terminator.
-            writer.write_all(&self.0 .0)?;
-            writer.write_all(&[0u8])?;
-        }
-        Ok(())
-    }
-
-    fn size_in_bytes(&self) -> u64 {
-        self.0.size_in_bytes()
-    }
-
-    fn alignment_in_bytes(&self) -> u64 {
-        N as u64
-    }
-}
-
-impl<T: SsbhWrite + binread::BinRead> SsbhWrite for Ptr64<T> {
+impl<P: Offset, T: SsbhWrite + BinRead<Args = ()>> SsbhWrite for Ptr<P, T> {
     fn ssbh_write<W: Write + Seek>(
         &self,
         writer: &mut W,
         data_ptr: &mut u64,
     ) -> std::io::Result<()> {
         // TODO: This is nearly identical to the relative pointer function.
-        let alignment = self.0.alignment_in_bytes();
-
-        // The data pointer must point past the containing type.
+        // The data pointer must point past the containing struct.
         let current_pos = writer.stream_position()?;
-        if *data_ptr <= current_pos {
+        if *data_ptr < current_pos + self.size_in_bytes() {
             *data_ptr = current_pos + self.size_in_bytes();
         }
 
-        // Calculate the absolute offset.
-        *data_ptr = round_up(*data_ptr, alignment);
-        write_u64(writer, *data_ptr)?;
+        match &self.0 {
+            Some(value) => {
+                let alignment = T::alignment_in_bytes();
 
-        // Write the data at the specified offset.
-        let pos_after_offset = writer.stream_position()?;
-        writer.seek(SeekFrom::Start(*data_ptr))?;
+                // The data pointer must point past the containing type.
+                let current_pos = writer.stream_position()?;
+                if *data_ptr < current_pos + self.size_in_bytes() {
+                    *data_ptr = current_pos + self.size_in_bytes();
+                }
 
-        self.0.ssbh_write(writer, data_ptr)?;
+                // Calculate the absolute offset.
+                *data_ptr = round_up(*data_ptr, alignment);
 
-        // Point the data pointer past the current write.
-        // Types with relative offsets will already increment the data pointer.
-        let current_pos = writer.stream_position()?;
-        if current_pos > *data_ptr {
-            *data_ptr = round_up(current_pos, alignment);
+                let offset = P::try_from(*data_ptr)
+                    .map_err(|_| std::io::Error::new(std::io::ErrorKind::Other, "oh no!"))?;
+                P::ssbh_write(&offset, writer, data_ptr)?;
+
+                // Write the data at the specified offset.
+                let pos_after_offset = writer.stream_position()?;
+                writer.seek(SeekFrom::Start(*data_ptr))?;
+
+                value.ssbh_write(writer, data_ptr)?;
+
+                // Point the data pointer past the current write.
+                // Types with relative offsets will already increment the data pointer.
+                let current_pos = writer.stream_position()?;
+                if current_pos > *data_ptr {
+                    *data_ptr = round_up(current_pos, alignment);
+                }
+
+                writer.seek(SeekFrom::Start(pos_after_offset))?;
+                Ok(())
+            }
+            None => {
+                P::default().ssbh_write(writer, data_ptr)?;
+                Ok(())
+            }
         }
-
-        writer.seek(SeekFrom::Start(pos_after_offset))?;
-        Ok(())
     }
 
     fn size_in_bytes(&self) -> u64 {
-        8
+        // TODO: Use the size_in_bytes already defined for P?
+        std::mem::size_of::<P>() as u64
     }
 }
 
@@ -399,11 +287,11 @@ impl<T: SsbhWrite + binread::BinRead> SsbhWrite for RelPtr64<T> {
     ) -> std::io::Result<()> {
         // The data pointer must point past the containing struct.
         let current_pos = writer.stream_position()?;
-        if *data_ptr <= current_pos {
+        if *data_ptr < current_pos + self.size_in_bytes() {
             *data_ptr = current_pos + self.size_in_bytes();
         }
 
-        write_rel_ptr_aligned(writer, &self.0, data_ptr, self.0.alignment_in_bytes())?;
+        write_rel_ptr_aligned(writer, &self.0, data_ptr, T::alignment_in_bytes())?;
 
         Ok(())
     }
@@ -413,37 +301,6 @@ impl<T: SsbhWrite + binread::BinRead> SsbhWrite for RelPtr64<T> {
     }
 }
 
-impl<T: SsbhWrite> SsbhWrite for Vec<T> {
-    fn ssbh_write<W: Write + Seek>(
-        &self,
-        writer: &mut W,
-        data_ptr: &mut u64,
-    ) -> std::io::Result<()> {
-        // The data pointer must point past the containing struct.
-        let current_pos = writer.stream_position()?;
-        if *data_ptr <= current_pos {
-            *data_ptr = current_pos + self.size_in_bytes();
-        }
-
-        for elem in self.iter() {
-            elem.ssbh_write(writer, data_ptr)?;
-        }
-
-        Ok(())
-    }
-
-    fn size_in_bytes(&self) -> u64 {
-        if self.is_empty() {
-            0
-        } else {
-            match self.first() {
-                Some(first) => self.len() as u64 * first.size_in_bytes(),
-                None => 0,
-            }
-        }
-    }
-}
-
 pub(crate) fn write_ssbh_header_and_data<W: Write + Seek>(
     writer: &mut W,
     data: &SsbhFile,
@@ -499,37 +356,82 @@ mod tests {
     // It's unclear if these rules are strictly required by the format or in game parsers,
     // but following these rules creates 1:1 export for all formats except NRPD.
 
-    // 1. Offsets are nonnegative.
-    // 2. Offsets point past the containing struct.
-    // 3. Offsets in array elements point past the containing array.
-    // 4. Offsets obey the alignment rules of the pointed to data's type.
-
     use super::*;
-    use crate::{SsbhEnum64, SsbhString, SsbhString8};
+    use crate::{Ptr16, Ptr32, Ptr64, SsbhEnum64, SsbhString, SsbhString8, hex_bytes};
     use binread::BinRead;
 
-    fn hex_bytes(hex: &str) -> Vec<u8> {
-        // Remove any whitespace used to make the tests more readable.
-        let no_whitespace: String = hex.chars().filter(|c| !c.is_whitespace()).collect();
-        hex::decode(no_whitespace).unwrap()
-    }
-
     #[test]
-    fn write_ptr() {
-        let value = Ptr64(5u64);
+    fn write_ptr16() {
+        let value = Ptr16::<u8>::new(5u8);
 
         let mut writer = Cursor::new(Vec::new());
         let mut data_ptr = 0;
         value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
-        assert_eq!(
-            *writer.get_ref(),
-            hex_bytes(
-                "08000000 00000000 
-                 05000000 00000000"
-            )
-        );
-        assert_eq!(16, data_ptr);
+        assert_eq!(*writer.get_ref(), hex_bytes("0200 05"));
+        assert_eq!(3, data_ptr);
+    }
+
+    #[test]
+    fn write_ptr32() {
+        let value = Ptr32::<u8>::new(5u8);
+
+        let mut writer = Cursor::new(Vec::new());
+        let mut data_ptr = 0;
+        value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
+
+        assert_eq!(*writer.get_ref(), hex_bytes("04000000 05"));
+        assert_eq!(5, data_ptr);
+    }
+
+    #[test]
+    fn write_null_ptr32() {
+        let value = Ptr32::<u8>::null();
+
+        let mut writer = Cursor::new(Vec::new());
+        let mut data_ptr = 0;
+        value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
+
+        assert_eq!(*writer.get_ref(), hex_bytes("00000000"));
+        assert_eq!(4, data_ptr);
+    }
+
+    #[test]
+    fn write_ptr64() {
+        let value = Ptr64::<u8>::new(5u8);
+
+        let mut writer = Cursor::new(Vec::new());
+        let mut data_ptr = 0;
+        value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
+
+        assert_eq!(*writer.get_ref(), hex_bytes("08000000 00000000 05"));
+        assert_eq!(9, data_ptr);
+    }
+
+    #[test]
+    fn write_ptr64_vec_u8() {
+        // Check that the alignment uses the inner type's alignment.
+        let value = Ptr64::new(vec![5u8]);
+
+        let mut writer = Cursor::new(Vec::new());
+        let mut data_ptr = 0;
+        value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
+
+        assert_eq!(*writer.get_ref(), hex_bytes("08000000 00000000 05"));
+        assert_eq!(9, data_ptr);
+    }
+
+    #[test]
+    fn write_ptr64_vec_u32() {
+        // Check that the alignment uses the inner type's alignment.
+        let value = Ptr64::new(vec![5u32]);
+
+        let mut writer = Cursor::new(Vec::new());
+        let mut data_ptr = 0;
+        value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
+
+        assert_eq!(*writer.get_ref(), hex_bytes("08000000 00000000 05000000"));
+        assert_eq!(12, data_ptr);
     }
 
     #[test]
@@ -672,10 +574,7 @@ mod tests {
         let mut data_ptr = 0;
         value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
-        assert_eq!(
-            *writer.get_ref(),
-            hex_bytes("01020304 05")
-        );
+        assert_eq!(*writer.get_ref(), hex_bytes("01020304 05"));
         assert_eq!(5, data_ptr);
     }
 
