@@ -1,12 +1,10 @@
-use std::io::{Read, Seek};
 use binread::{BinRead, BinReaderExt, BinResult, ReadOptions};
-use ssbh_write_derive::SsbhWrite;
-
-#[cfg(feature = "derive_serde")]
+#[cfg(feature = "serde")]
 use serde::{Deserialize, Serialize};
+use ssbh_write::SsbhWrite;
+use std::io::{Read, Seek};
 
-
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[derive(BinRead, Debug, SsbhWrite)]
 pub struct MeshItem {
     pub mesh_index: i32,
@@ -16,7 +14,7 @@ pub struct MeshItem {
 }
 
 /// Mesh adjacency data for model.adjb files.
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[derive(BinRead, Debug, SsbhWrite)]
 pub struct Adj {
     pub count: u32,
@@ -27,8 +25,10 @@ pub struct Adj {
     pub buffer: Vec<i16>,
 }
 
+// TODO: This could be a shared function in lib.rs.
 fn read_to_end<R: Read + Seek>(reader: &mut R, _ro: &ReadOptions, _: ()) -> BinResult<Vec<i16>> {
     let mut buf = Vec::new();
+    // TODO: Read until EOF?
     while let Ok(v) = reader.read_le::<i16>() {
         buf.push(v);
     }
