@@ -9,15 +9,15 @@ use crate::SsbhByteBuffer;
 use crate::SsbhString;
 use crate::Vector3;
 
-#[cfg(feature = "derive_serde")]
-use serde::{Deserialize, Serialize};
-use ssbh_write_derive::SsbhWrite;
-
 use binread::BinRead;
+#[cfg(feature = "serde")]
+use serde::{Deserialize, Serialize};
+use ssbh_write::SsbhWrite;
 
 /// The vertex buffers and associated geometric data for a mesh.
 /// Compatible with file version 1.8, 1.9, and 1.10.
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite)]
 #[ssbhwrite(pad_after = 16, align_after = 8)]
 pub struct Mesh {
@@ -31,6 +31,7 @@ pub struct Mesh {
     pub buffer_sizes: SsbhArray<u32>,
     pub polygon_index_size: u64,
     /// The shared buffers for vertex attribute data such as positions and normals for all the [MeshObject] in [objects](#structfield.objects).
+    /// Each [MeshObject] only defines attribute access information for up to 4 buffers.
     pub vertex_buffers: SsbhArray<SsbhByteBuffer>,
     /// The shared buffer for vertex indices for all the [MeshObject] in [objects](#structfield.objects).
     pub index_buffer: SsbhByteBuffer,
@@ -41,8 +42,9 @@ pub struct Mesh {
     pub rigging_buffers: SsbhArray<MeshRiggingGroup>,
 }
 
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
+#[derive(BinRead, Debug, SsbhWrite, PartialEq, Eq)]
 pub struct MeshAttributeV8 {
     pub usage: AttributeUsageV8,
     pub data_type: AttributeDataTypeV8,
@@ -52,8 +54,9 @@ pub struct MeshAttributeV8 {
     pub sub_index: u32,
 }
 
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
+#[derive(BinRead, Debug, SsbhWrite, PartialEq, Eq)]
 pub struct MeshAttributeV9 {
     pub usage: AttributeUsageV9,
     pub data_type: AttributeDataTypeV8,
@@ -65,8 +68,9 @@ pub struct MeshAttributeV9 {
     pub attribute_names: SsbhArray<SsbhString>,
 }
 
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
+#[derive(BinRead, Debug, SsbhWrite, PartialEq, Eq)]
 pub struct MeshAttributeV10 {
     pub usage: AttributeUsageV9,
     pub data_type: AttributeDataTypeV10,
@@ -78,7 +82,8 @@ pub struct MeshAttributeV10 {
     pub attribute_names: SsbhArray<SsbhString>,
 }
 
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite, Clone, Copy)]
 pub struct BoundingInfo {
     pub bounding_sphere: BoundingSphere,
@@ -86,7 +91,8 @@ pub struct BoundingInfo {
     pub oriented_bounding_box: OrientedBoundingBox,
 }
 
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite, Clone, Copy)]
 pub struct BoundingSphere {
     pub center: Vector3,
@@ -95,14 +101,16 @@ pub struct BoundingSphere {
 
 /// A region of 3d space that contains a set of points.
 /// This is equivalent to an axis-aligned bounding box (abbreviated AABB) for the XYZ axes.
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite, Clone, Copy)]
 pub struct BoundingVolume {
     pub min: Vector3,
     pub max: Vector3,
 }
 
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite, Clone, Copy)]
 pub struct OrientedBoundingBox {
     pub center: Vector3,
@@ -110,7 +118,8 @@ pub struct OrientedBoundingBox {
     pub size: Vector3,
 }
 
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite, Copy, Clone)]
 #[ssbhwrite(pad_after = 6)]
 pub struct RiggingFlags {
@@ -119,7 +128,8 @@ pub struct RiggingFlags {
     pub unk1: u8,
 }
 
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite)]
 #[br(import(major_version: u16, minor_version: u16))]
 pub struct MeshBoneBuffer {
@@ -128,7 +138,8 @@ pub struct MeshBoneBuffer {
     pub data: VertexWeights,
 }
 
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite)]
 #[br(import(major_version: u16, minor_version: u16))]
 pub enum VertexWeights {
@@ -144,7 +155,8 @@ pub enum VertexWeights {
 
 /// Vertex skinning data for the vertices for the [MeshObject]
 /// determined by [mesh_object_name](#structfield.mesh_object_name) and [mesh_object_sub_index](#structfield.mesh_object_sub_index).
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite)]
 #[br(import(major_version: u16, minor_version: u16))]
 pub struct MeshRiggingGroup {
@@ -155,7 +167,8 @@ pub struct MeshRiggingGroup {
     pub buffers: SsbhArray<MeshBoneBuffer>,
 }
 
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite)]
 #[br(import(major_version: u16, minor_version: u16))]
 pub enum MeshAttributes {
@@ -169,24 +182,26 @@ pub enum MeshAttributes {
     AttributesV10(SsbhArray<MeshAttributeV10>),
 }
 
-/// The element type for the vertex skin weights stored in version 1.10 byte buffers.
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+/// The type of array element for the vertex skin weights stored in the [SsbhByteBuffer] for [VertexWeights::VertexWeightsV8] and [VertexWeights::VertexWeightsV9].
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite)]
 pub struct VertexWeightV8 {
     pub vertex_index: u32,
     pub vertex_weight: f32,
 }
 
-/// The element type for the vertex skin weights stored in the [SsbhByteBuffer] for [VertexWeights::VertexWeightsV10].
+/// The type of array element for the vertex skin weights stored in the [SsbhByteBuffer] for [VertexWeights::VertexWeightsV10].
 #[derive(BinRead, Debug)]
 pub struct VertexWeightV10 {
     pub vertex_index: u16,
     pub vertex_weight: f32,
 }
 
-/// A vertex collection identified by its [name](#structfield.name) and [sub_index](#structfield.sub_index).
+/// An indexed vertex collection identified by its [name](#structfield.name) and [sub_index](#structfield.sub_index).
 /// In addition to organizing the model into logical components, material and rigging data are assigned per [MeshObject].
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite)]
 #[br(import(major_version: u16, minor_version: u16))]
 pub struct MeshObject {
@@ -208,12 +223,16 @@ pub struct MeshObject {
     pub unk2: u32, // number of indices per face (always 3)?
     pub vertex_buffer0_offset: u32,
     pub vertex_buffer1_offset: u32,
-    pub final_buffer_offset: u32,
-    pub buffer_index: u32, // always 0?
+    pub vertex_buffer2_offset: u32,
+    pub vertex_buffer3_offset: u32,
+    /// The stride in bytes for the first buffer in [vertex_buffers](struct.Mesh.html#structfield.vertex_buffers).
     pub stride0: u32,
+    /// The stride in bytes for the second buffer in [vertex_buffers](struct.Mesh.html#structfield.vertex_buffers).
     pub stride1: u32,
-    pub unk6: u32, // set to 32 for version 1.8 and 0 otherwise
-    pub unk7: u32, // always 0
+    /// The stride in bytes for the third buffer in [vertex_buffers](struct.Mesh.html#structfield.vertex_buffers).
+    pub stride2: u32,
+    /// The stride in bytes for the fourth buffer in [vertex_buffers](struct.Mesh.html#structfield.vertex_buffers).
+    pub stride3: u32,
     /// The byte offset for the start of this object's vertex indices in the [index_buffer](struct.Mesh.html#structfield.index_buffer).
     /// The number of bytes to read is determined by [draw_element_type](#structfield.draw_element_type) and [vertex_index_count](#structfield.vertex_index_count).
     pub index_buffer_offset: u32,
@@ -231,8 +250,9 @@ pub struct MeshObject {
 }
 
 /// Possible values for [draw_element_type](struct.MeshObject.html#structfield.draw_element_type).
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
-#[derive(BinRead, Debug, Clone, Copy, PartialEq)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
+#[derive(BinRead, Debug, Clone, Copy, PartialEq, Eq)]
 #[br(repr(u32))]
 pub enum DrawElementType {
     /// Vertex indices stored as [u16].
@@ -242,8 +262,9 @@ pub enum DrawElementType {
 }
 
 /// Possible values for [rigging_type](struct.MeshObject.html#structfield.rigging_type).
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
-#[derive(BinRead, Debug, Clone, Copy, PartialEq)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
+#[derive(BinRead, Debug, Clone, Copy, PartialEq, Eq)]
 #[br(repr(u32))]
 pub enum RiggingType {
     /// Vertices are parented to a parent bone and inherit the parent's transforms.
@@ -255,8 +276,9 @@ pub enum RiggingType {
 
 /// The data type and component count for the attribute's data.
 /// This determines the stride and offset between attributes.
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
-#[derive(BinRead, Debug, Clone, Copy, PartialEq)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
+#[derive(BinRead, Debug, Clone, Copy, PartialEq, Eq)]
 #[br(repr(u32))]
 pub enum AttributeDataTypeV10 {
     /// 3 component (xyz or rgb) vector of [f32].
@@ -275,8 +297,9 @@ pub enum AttributeDataTypeV10 {
 
 /// The data type and component count for the attribute's data.
 /// This determines the stride and offset between attributes.
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
-#[derive(BinRead, Debug, Clone, Copy, PartialEq)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
+#[derive(BinRead, Debug, Clone, Copy, PartialEq, Eq)]
 #[br(repr(u32))]
 pub enum AttributeDataTypeV8 {
     /// 3 component (xyz or rgb) vector of [f32].
@@ -295,8 +318,9 @@ pub enum AttributeDataTypeV8 {
 /// Attributes with an identical usage should each have a unique [sub_index](struct.MeshAttributeV10.html#structfield.sub_index).
 /// Smash Ultimate also considers [name](struct.MeshAttributeV10.html#structfield.name) and
 /// [attribute_names](struct.MeshAttributeV10.html#structfield.attribute_names) when determing the usage in some cases.
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
-#[derive(BinRead, Debug, Clone, Copy, PartialEq)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
+#[derive(BinRead, Debug, Clone, Copy, PartialEq, Eq)]
 #[br(repr(u32))]
 pub enum AttributeUsageV9 {
     Position = 0,
@@ -309,8 +333,9 @@ pub enum AttributeUsageV9 {
 
 /// Determines how the attribute data will be used by the shaders for [Mesh] version 1.8.
 /// Attributes with an identical usage should each have a unique [sub_index](struct.MeshAttributeV8.html#structfield.sub_index).
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
-#[derive(BinRead, Debug, Clone, Copy, PartialEq)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
+#[derive(BinRead, Debug, Clone, Copy, PartialEq, Eq)]
 #[br(repr(u32))]
 pub enum AttributeUsageV8 {
     Position = 0,
