@@ -4,14 +4,14 @@
 
 use crate::{SsbhArray, SsbhString, SsbhString8};
 use binread::BinRead;
-use ssbh_write_derive::SsbhWrite;
-
-#[cfg(feature = "derive_serde")]
+#[cfg(feature = "serde")]
 use serde::{Deserialize, Serialize};
+use ssbh_write::SsbhWrite;
 
 /// A required vertex attribute.
 /// The [name](#structfield.name) and [attribute_name](#structfield.attribute_name) should match the values for a corresponding [MeshAttributeV10][crate::formats::mesh::MeshAttributeV10].
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite)]
 pub struct VertexAttribute {
     pub name: SsbhString,
@@ -19,17 +19,20 @@ pub struct VertexAttribute {
 }
 
 /// A required material parameter. The [param_id](#structfield.param_id) and [parameter_name](#structfield.parameter_name) match one of the variants in [ParamId](crate::formats::matl::ParamId).
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite)]
 #[ssbhwrite(pad_after = 8)]
 pub struct MaterialParameter {
     // TODO: These values are identical to the matl ones but there are some missing variants.
     pub param_id: u64,
+    #[br(pad_after = 8)]
     pub parameter_name: SsbhString8,
 }
 
 /// Describes the shaders used for each of the stages in the rendering pipeline.
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite)]
 pub struct ShaderStages {
     pub vertex_shader: SsbhString,
@@ -42,11 +45,12 @@ pub struct ShaderStages {
 
 /// Describes the name and associated information for a set of compiled shaders linked into a program.
 /// Each [ShaderProgramV0] has a corresponding shader program object in the underlying rendering API such as OpenGL, Vulkan, etc.
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite)]
 pub struct ShaderProgramV0 {
     /// The unique identifier of the shader program, including its [render_pass](#structfield.render_pass).
-    pub name: SsbhString8,
+    pub name: SsbhString,
     /// Programs are grouped into passes to determine the render order.
     /// Possible values for Smash Ultimate are "nu::Final", "nu::Opaque", "nu::Sort", "nu::Near", and "nu::Far".
     pub render_pass: SsbhString,
@@ -58,11 +62,12 @@ pub struct ShaderProgramV0 {
 
 /// Describes the name and associated information for a set of compiled shaders linked into a program.
 /// Each [ShaderProgramV1] has a corresponding shader program object in the underlying rendering API such as OpenGL, Vulkan, etc.
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite)]
 pub struct ShaderProgramV1 {
     /// The unique identifier of the shader program, including its [render_pass](#structfield.render_pass).
-    pub name: SsbhString8,
+    pub name: SsbhString,
     /// Programs are grouped into passes to determine the render order.
     /// Possible values for Smash Ultimate are "nu::Final", "nu::Opaque", "nu::Sort", "nu::Near", and "nu::Far".
     pub render_pass: SsbhString,
@@ -74,14 +79,16 @@ pub struct ShaderProgramV1 {
     pub material_parameters: SsbhArray<MaterialParameter>,
 }
 
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite)]
 pub struct UnkItem {
     pub name: SsbhString,
     pub unk1: SsbhArray<SsbhString>,
 }
 
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite)]
 #[br(import(major_version: u16, minor_version: u16))]
 pub enum ShaderPrograms {
@@ -93,7 +100,8 @@ pub enum ShaderPrograms {
 
 /// A shader effects library that describes shader programs and their associated inputs.
 /// Compatible with file version 1.0 and 1.1.
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite)]
 pub struct Nufx {
     pub major_version: u16,
