@@ -1,12 +1,12 @@
 use crate::{SsbhArray, SsbhByteBuffer, SsbhString};
 use binread::BinRead;
-use ssbh_write_derive::SsbhWrite;
-
-#[cfg(feature = "derive_serde")]
+#[cfg(feature = "serde")]
 use serde::{Deserialize, Serialize};
+use ssbh_write::SsbhWrite;
 
 #[repr(u32)]
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, Clone, Copy)]
 #[br(repr(u32))]
 pub enum ShaderType {
@@ -16,7 +16,8 @@ pub enum ShaderType {
     Compute = 5,
 }
 
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite)]
 pub struct Shader {
     pub name: SsbhString,
@@ -30,7 +31,8 @@ pub struct Shader {
 
 /// A compiled shader container.
 /// Compatible with file version 1.2.
-#[cfg_attr(feature = "derive_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite)]
 pub struct Shdr {
     pub major_version: u16,
